@@ -16,13 +16,14 @@
 package collector
 
 import (
-	"bufio"
 	"fmt"
 	"io"
+	"io/ioutil"
 	"os"
 	"strconv"
 	"strings"
 
+	"github.com/go-kit/kit/log"
 	"github.com/prometheus/client_golang/prometheus"
 )
 
@@ -51,10 +52,15 @@ const (
 	tcpListen
 	// TCP_CLOSING
 	tcpClosing
+	// TCP_RX_BUFFER
+	tcpRxQueuedBytes
+	// TCP_TX_BUFFER
+	tcpTxQueuedBytes
 )
 
 type tcpStatCollector struct {
-	desc typedDesc
+	desc   typedDesc
+	logger log.Logger
 }
 
 func init() {
@@ -62,13 +68,14 @@ func init() {
 }
 
 // NewTCPStatCollector returns a new Collector exposing network stats.
-func NewTCPStatCollector() (Collector, error) {
+func NewTCPStatCollector(logger log.Logger) (Collector, error) {
 	return &tcpStatCollector{
 		desc: typedDesc{prometheus.NewDesc(
 			prometheus.BuildFQName(namespace, "tcp", "connection_states"),
 			"Number of connection states.",
 			[]string{"state"}, nil,
 		), prometheus.GaugeValue},
+		logger: logger,
 	}, nil
 }
 
@@ -108,32 +115,48 @@ func getTCPStats(statsFile string) (map[tcpConnectionState]float64, error) {
 }
 
 func parseTCPStats(r io.Reader) (map[tcpConnectionState]float64, error) {
-	var (
-		tcpStats = map[tcpConnectionState]float64{}
-		scanner  = bufio.NewScanner(r)
-	)
+	tcpStats := map[tcpConnectionState]float64{}
+	contents, err := ioutil.ReadAll(r)
+	if err != nil {
+		return nil, err
+	}
 
-	for scanner.Scan() {
-		parts := strings.Fields(scanner.Text())
+	for _, line := range strings.Split(string(contents), "\n")[1:] {
+		parts := strings.Fields(line)
 		if len(parts) == 0 {
 			continue
 		}
-		if len(parts) < 4 {
-			return nil, fmt.Errorf("invalid TCP stats line: %q", scanner.Text())
+		if len(parts) < 5 {
+			return nil, fmt.Errorf("invalid TCP stats line: %q", line)
 		}
 
-		if strings.HasPrefix(parts[0], "sl") {
-			continue
+		qu := strings.Split(parts[4], ":")
+		if len(qu) < 2 {
+			return nil, fmt.Errorf("cannot parse tx_queues and rx_queues: %q", line)
 		}
+
+		tx, err := strconv.ParseUint(qu[0], 16, 64)
+		if err != nil {
+			return nil, err
+		}
+		tcpStats[tcpConnectionState(tcpTxQueuedBytes)] += float64(tx)
+
+		rx, err := strconv.ParseUint(qu[1], 16, 64)
+		if err != nil {
+			return nil, err
+		}
+		tcpStats[tcpConnectionState(tcpRxQueuedBytes)] += float64(rx)
+
 		st, err := strconv.ParseInt(parts[3], 16, 8)
 		if err != nil {
 			return nil, err
 		}
 
 		tcpStats[tcpConnectionState(st)]++
+
 	}
 
-	return tcpStats, scanner.Err()
+	return tcpStats, nil
 }
 
 func (st tcpConnectionState) String() string {
@@ -160,6 +183,10 @@ func (st tcpConnectionState) String() string {
 		return "listen"
 	case tcpClosing:
 		return "closing"
+	case tcpRxQueuedBytes:
+		return "rx_queued_bytes"
+	case tcpTxQueuedBytes:
+		return "tx_queued_bytes"
 	default:
 		return "unknown"
 	}
